package com.tinysine.tosrblue.io;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.UUID;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Handler;
import android.preference.PreferenceManager;

import com.tinysine.tosrblue.ATConfigActivity;
import com.tinysine.tosrblue.Bluemd;
import com.tinysine.tosrblue.Util;
import com.tinysine.tosrblue.command.ATCommandInput;
import com.tinysine.tosrblue.command.ATCommandOutput;

public class ATConnectThread extends Thread {

	protected BluetoothDevice dev = null;
	protected BluetoothSocket sock = null;
	protected boolean stop = false;
	protected static final UUID MY_UUID = UUID
			.fromString("00001101-0000-1000-8000-00805F9B34FB");
	protected InputStream in = null;
	protected OutputStream out = null;
	Bluemd openWnn = null;
	protected ArrayList<ATCommandOutput> cmds = null;
	ATCommandInput cmdInput = null;

	protected Context context;
	Handler handler;

	public ATConnectThread(BluetoothDevice dev,
			ArrayList<ATCommandOutput> cmds, Bluemd openWnn, Context context,
			Handler handler) {
		this.dev = dev;
		this.cmds = cmds;
		this.openWnn = openWnn;
		this.context = context;
		this.handler = handler;
		setState(Util.BT_STATE, Util.STATE_NONE);
	}

	public void startDevice() throws IOException, InterruptedException {
		setState(Util.BT_STATE, Util.STATE_CONNECTING);
		sock = dev.createRfcommSocketToServiceRecord(MY_UUID);
		sock.connect();
		in = sock.getInputStream();
		out = sock.getOutputStream();
		setState(Util.BT_STATE, Util.STATE_CONNECTED);
	}

	public void run() {
		try {
			startDevice();
			cmdInput = new ATCommandInput(in, "", "", handler);
			runCommandInput(cmdInput);
			sleep(1000);
			while (cmdInput.isAlive()) {
				sleep(500);
			}
			while (aCommand.isAlive()) {
				sleep(500);
			}
		} catch (Exception e) {
			setState(Util.BT_STATE, Util.STATE_NONE);
		} finally {
			close();
		}
	}

	private int mState = 0;

	private synchronized void setState(int what, int state) {
		mState = state;
		handler.obtainMessage(what, state, -1).sendToTarget();
	}

	public void cancel() {
		stop = true;
		if (cmdInput != null && cmdInput.isAlive()) {
			cmdInput.cancle();
		}
		close();
	}

	public void SendCMDS(ATCommandOutput cmd) {
		try {
			runCommand(cmd);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void close() {
		try {
			stop = true;
			sock.close();
		} catch (Exception e) {
		}
	}

	public static ATCommandOutput getCopy(ATCommandOutput cmd)
			throws IllegalArgumentException, SecurityException,
			InstantiationException, IllegalAccessException,
			InvocationTargetException, NoSuchMethodException {
		return cmd.getClass().getConstructor(cmd.getClass()).newInstance(cmd);
	}

	// 开起发送
	public void runCommand(ATCommandOutput cmd) throws InterruptedException {
		cmd.setOut(out);
		cmd.setConnectThread(this);
		cmd.start();

	}

	// 开启接收
	public void runCommandInput(ATCommandInput cmd) throws InterruptedException {
		cmd.setIn(in);
		cmd.start();
	}

	class SendCommand extends Thread {
		ATCommandOutput cmd;

		SendCommand(ATCommandOutput cmd) {
			this.cmd = cmd;
		}

		@Override
		public void run() {
			// TODO Auto-generated method stub
			SendCMDS(cmd);
			super.run();
		}
	}

	SendCommand aCommand = null;

	public void Sends(ATCommandOutput cmd) {
		if (aCommand == null || !aCommand.isAlive()) {
			aCommand = new SendCommand(cmd);
			aCommand.setPriority(Thread.NORM_PRIORITY + 1);
			aCommand.start();
		}
	}

	public void GetCmds() {
		cmds.clear();
		SharedPreferences prefs = PreferenceManager
				.getDefaultSharedPreferences(context);
		cmds = ATConfigActivity.getObdCommands(prefs);
		String cmdString = prefs.getString(ATConfigActivity.STRING_SUFFIX1,
				null);
		if (cmdString != null && !"".equals(cmdString)) {
			cmds.add(new ATCommandOutput(cmdString, cmdString, cmdString));
		}
	}

	/**
	 * Return the current connection data.
	 */
	public synchronized String getData() {
		if (cmdInput != null && cmdInput.isAlive()) {
			return cmdInput.bytesToHexToTwoString();
		}
		return null;
	}

	public synchronized byte getByteData() {

		if (cmdInput != null && cmdInput.isAlive()) {

			return cmdInput.Getbyte();
		}
		return 0;
	}

	/**
	 * Return the current connection state.
	 */
	public synchronized int getState1() {
		return mState;
	}

}
