package com.tinysine.tosrblue.io;

import java.io.StringWriter;
import java.util.ArrayList;

import android.app.Service;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Binder;
import android.os.Handler;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.widget.Toast;

import com.tinysine.tosrblue.ATConfigActivity;
import com.tinysine.tosrblue.Bluemd;
import com.tinysine.tosrblue.Util;
import com.tinysine.tosrblue.command.ATCommandOutput;


public class ATService  extends Service{

	private final IBinder binder = new ATServiceBinder();
	
	public ATConnectThread connectThread = null;

	public static final int COMMAND_ERROR_NOTIFY = 2;
	public static final int CONNECT_ERROR_NOTIFY = 3;
	public static final int OBD_SERVICE_RUNNING_NOTIFY = 4;
	public static final int OBD_SERVICE_ERROR_NOTIFY = 5;

	@Override
	public IBinder onBind(Intent intent) {
		// TODO Auto-generated method stub
		return binder;
	}
	
	public class ATServiceBinder extends Binder {
		ATService getService() {
			return ATService.this;
		}
	}
	
	public void onCreate() {
		super.onCreate();
	}
	public void onDestroy() {
		super.onDestroy();
		stopService();
		stopSelf();
	}
	public boolean isRunning() {
		if (connectThread == null) {
			return false;
		}
		return connectThread.isAlive();
	}
	BluetoothDevice dev=null;
	Handler handler;
	public boolean startService( Bluemd openWnn ,String address
			,Handler handler) {
		this.handler =handler;
		if (connectThread != null && connectThread.isAlive()) {
			return true;
		}
		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
		final BluetoothAdapter mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
    	if (mBluetoothAdapter == null) {
    		Toast.makeText(this, "This device does not support bluetooth", Toast.LENGTH_LONG).show();
    		stopSelf();
        	return false;
        }
    	if (address == null || "".equals(address)) {
			stopSelf();
			return false;
		}

        ArrayList<ATCommandOutput> cmds = ATConfigActivity.getObdCommands(prefs);
        String cmdString = prefs.getString(ATConfigActivity.STRING_SUFFIX1, null);
        if(cmdString!=null&&!"".equals(cmdString)){
        	 cmds.add(new ATCommandOutput(cmdString,cmdString,cmdString));
        }
		 dev = mBluetoothAdapter.getRemoteDevice(address);
		mBluetoothAdapter.cancelDiscovery();
		connectThread = new ATConnectThread(dev,cmds,openWnn,this,handler);
		new ObdReaderServiceWorkerThread(connectThread).start();
		return true;
	}
	public boolean stopService() {
		if (connectThread == null) {
			return true;
		}
		while (connectThread.isAlive()) {
			try {
				connectThread.cancel();
				connectThread.join(300);
			} catch (InterruptedException e) {
		    	StringWriter strw = new StringWriter();
				notifyMessage(e.getMessage(),strw.toString(), OBD_SERVICE_ERROR_NOTIFY);
			}
		}
		connectThread.close();
		stopSelf();
		return true;
	}
	public void notifyMessage(String msg, String longMsg, int notifyId) {
	}
	
	public String getDevname() {
		if (dev!=null) {
			return dev.getName();
		}
		return "null"; 
	}
	public String getDataMap() {
		if (connectThread != null) {
			return connectThread.getData();
		}
		return null;
	}
	
	public String getDevAddress() {
		if (dev != null) {
			return dev.getAddress();
		}
		return "null";
	}
	
	public byte getByteDataMap() {

		if (connectThread != null) {
			
			return connectThread.getByteData();
		}
		return 0;
	}
	
	
	public String cleanDataMap() {
		return null;
	}
	public class ObdReaderServiceBinder extends Binder {
		ATService getService() {
			return ATService.this;
		}
	}
	public void Send(ATCommandOutput cmd) {
		if (connectThread != null && connectThread.isAlive()) {
			connectThread.Sends(cmd);
		}
	}
	
	private class ObdReaderServiceWorkerThread extends Thread {
		ATConnectThread t = null;
		public ObdReaderServiceWorkerThread(ATConnectThread t) {
			this.t = t;
		}
		public void run() {
			try {
				t.start();
				t.join();
			} catch (Exception e) {
			} finally {
				setState(Util.BT_STATE, Util.STATE_NONE);
				stopSelf();
			}
		}
	}
	
	private synchronized void setState(int what, int state) {
		handler.obtainMessage(what, state, -1).sendToTarget();
	}
}
